/*
   Count the number of bytes in one file called "data".

   This version memory maps the data file using MappedByteBuffer
   and the map() method in FileChannel.

   The data file is larger than the 2GB mapping limit, so we need
   to map the file in two segments.
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
//import java.nio.MappedByteBuffer;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

public class CountBytesMemoryMappedFile
{
   public static void main(String[] args) throws IOException, FileNotFoundException
   {
      // Memory map the first 2GB segment of the file.
    //FileChannel fc = new FileInputStream("data").getChannel();
      FileChannel fc = new RandomAccessFile("data", "r").getChannel();
      long fileSize = fc.size();
      ByteBuffer mbb1 = fc.map(FileChannel.MapMode.READ_ONLY, 0, Integer.MAX_VALUE);

      final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");
      System.out.println(ft.format(new Date()) + " Count bytes in Memory Mapped File");
      Instant start = Instant.now();
      long byteCounter = 0;

      for (int i = 0; i < Integer.MAX_VALUE - 4096; i += 4096) // jump through virtual memory pages
      {
         mbb1.get(i); // touch page and cause a page fault (try commenting this out!)
         byteCounter += 4096;
      }

      // Memory map the second segment of the file.
      int restOfFile = (int)(fileSize - Integer.MAX_VALUE - 1);
      ByteBuffer mbb2 = fc.map(FileChannel.MapMode.READ_ONLY, 1L + Integer.MAX_VALUE, restOfFile);

      int i = 0;
      for (; i < restOfFile; i += 4096)
      {
         mbb2.get(i); // touch page and cause a page fault (try commenting this out!)
         byteCounter += 4096L;
      }
      byteCounter += 4096 - (i - restOfFile); // count the last partial page

      Instant stop = Instant.now();
      long time = Duration.between(start, stop).toMillis();
      System.out.printf("%s %,d bytes read from file.\n", ft.format(new Date()), byteCounter);
      System.out.printf("elapsed time: %,dms\n", time);
      System.out.println( "hasArray() ==> " + mbb1.hasArray() );
      System.out.println( "isDirect() ==> " + mbb1.isDirect() );
      System.out.println( mbb1 );
      System.out.println( mbb2 );
      long capacity1  = mbb1.capacity();
      long capacity2  = mbb2.capacity();
      System.out.printf("%,d + %,d = %,d\n", capacity1, capacity2, capacity1+capacity2);
      fc.close();
      System.out.print("Sleeping...");
      while(true){try{Thread.sleep(1000);}catch(Exception e){}}
   }
}
