/*
   Count the number of lines in one file called "data".

   This version memory maps the data file using MappedByteBuffer
   and the map() method in FileChannel.

   The data file is larger than the 2GB mapping limit, so we need
   to map the file in two segments.
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
//import java.nio.MappedByteBuffer;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

public class CountLinesMemoryMappedFile
{
   public static void main(String[] args) throws IOException, FileNotFoundException
   {
      // Memory map the first 2GB segment of the file.
    //FileChannel fc = new FileInputStream("data").getChannel();
      FileChannel fc = new RandomAccessFile("data", "r").getChannel();
      long fileSize = fc.size();
      ByteBuffer mbb = fc.map(FileChannel.MapMode.READ_ONLY, 0, Integer.MAX_VALUE);

      final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");
      System.out.println(ft.format(new Date()) + " Count lines in Memory Mapped File");
      Instant start = Instant.now();
      long byteCounter = 0;
      long lineCounter = 1;

      for (int i = 0; i < Integer.MAX_VALUE; ++i)
      {
         ++byteCounter;
         if ( '\n' == mbb.get() )
         {
            ++lineCounter;
         }
      }

      // Memory map the second segment of the file.
      int restOfFile = (int)(fileSize - Integer.MAX_VALUE - 1);
      mbb = fc.map(FileChannel.MapMode.READ_ONLY, 1L + Integer.MAX_VALUE, restOfFile);

      for (int i = 0; i < restOfFile; ++i)
      {
         ++byteCounter;
         if ( '\n' == mbb.get() )
         {
            ++lineCounter;
         }
      }
      Instant stop = Instant.now();
      long time = Duration.between(start, stop).toMillis();
      System.out.printf("%s %,d bytes read from file.\n", ft.format(new Date()), byteCounter);
      System.out.printf("%s %,d lines read from file.\n", ft.format(new Date()), lineCounter);
      System.out.printf("elapsed time: %,dms\n", time);
      fc.close();
   }
}
