/*



*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;

public class CountLines_v4_File
{
   private static final int BUFFERSIZE = 16 * 8192;

   public static void main(String[] args) throws IOException
   {
      final InputStream is = new FileInputStream("data");

      byte[] c = new byte[BUFFERSIZE];
      int n;

      final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");
      System.out.println(ft.format(new Date()) + " Count lines in File ver 4");
      Instant start = Instant.now();
      long byteCounter = 0;
      long lineCounter = 1;
      long crCounter = 0;      // check for ascii CR characters
      long eofCounter = 0;     // check for ascii eof characters
      long ctrlCounter = 0;    // check for ascii control characters
      long binaryCounter = 0;  // check for non-ascii binary bytes

      while ( (n = is.read(c)) != -1 )
      {
         byteCounter += n;
         for (int i = 0; i < n; ++i)
         {
            if ( '\n' == c[i] )
            {
               ++lineCounter;
            }
            else if ( 0x0D == c[i] )
            {
               ++crCounter;
               //System.out.printf("line %,d\n", lineCounter);
            }
            else if ( 0x0C == c[i] ) // 0x09 and 0x0C
            {
               ++eofCounter;
               //System.out.printf("line %,d\n", lineCounter);
            }
            else if ( 0x1F >= c[i] )
            {
               ++ctrlCounter;
               //System.out.printf("line %,d\n", lineCounter);
            }
            else if ( 0x80 <= c[i] )
            {
               ++binaryCounter;
               //System.out.printf("line %,d\n", lineCounter);
            }
         }
      }

      Instant stop = Instant.now();
      long time = Duration.between(start, stop).toMillis();
      System.out.printf("%s %,d bytes read from file.\n",
                        ft.format(new Date()), byteCounter);
      System.out.printf("%s %,d lines read from file.\n",
                        ft.format(new Date()), lineCounter);
      System.out.printf("%s %,d carriage returns read from file.\n",
                        ft.format(new Date()), crCounter);
      System.out.printf("%s %,d eof characters read from all files.\n",
                        ft.format(new Date()), eofCounter);
      System.out.printf("%s %,d ascii control characters read from file.\n",
                        ft.format(new Date()), ctrlCounter);
      System.out.printf("%s %,d non-ascii bytes read from file.\n",
                        ft.format(new Date()), binaryCounter);
      System.out.printf("elapsed time: %,dms\n", time);
      is.close();
   }
}
